(*****************************************************************************)
(*                                                                           *)
(*            Set .BDY paths & LIBRARY name for Ycode interpreter            *)
(*                   (c) 1990 Rumen Christov, George Petrov                  *)
(*                                                                           *)
(*  Usage: BLpath [BDY=<paths>] [LIB=<library name>] [?]                     *)
(*                                                                           *)
(*****************************************************************************)

program BLpath;                                                        {$I-,R-}
  (*){*) uses UniPascal; {*}

  const YcdOffs = $00;           { offset of Ycode interpreter version number }
        LIBOffs = $C2;                            { offset of LIBname pointer }
        BDYOffs = $C4;                          { offset of BDY paths pointer }
        PTHOffs = $C8;
        InvPrm  = 'Invalid parameter: ';

  type ba = packed array [0..255] of byte;
       bap = ^ba;

{       pathrec=packed record
          addr1, addr2 : bap;
          len1,len2 : shortcard
         end; }

  var SYSpage:  bap;
      wrk1,wrk2 : bap;
      x,SpcNeeded,
      Yversion: cardinal;          { Ycode versions < 1.50 can't process this }
      hasBDY,
      hasLIB:   boolean;
      Allowed:  set of char;
{     paths:pathrec; }
      OldLib,
      OldBDY,
      BDYpaths,
      LIBname:  string;

  (*)
  procedure MakeResident;
    code $5f, $d, 3, $3f, $2c, $39; { int $2c; rts }

  procedure UpCaseS(s: string);
    code $5f, $34, $83;

  {*)
  procedure UpCaseS(var s: string);
    var i: byte;
  begin
    for i:= 1 to length(s) do s[i]:= UpCase(s[i]);
  end (* UpCaseS *);
  {*}

  function blp:pointer; {$L blp.obj} external;

  procedure HelpUser;
  begin
    writeln('usage: BLpath [LIB=<Library name>] [BDY=<paths>] [?]');
    writeln('paths must be separated by semicolon');
    halt(1);
  end { HelpUser };

  procedure PutString( s:string; o:cardinal; var x:cardinal );
  var p:bap;
  begin { PutString }
    insert(#0, s, length(s) +1);
    p:=addr(wrk1^[x]);
    move(s[1], p^, length(s));
    {$w-} move(p, SYSpage^[o], 2);
    inc(x,length(s));
  end { PutString };

  procedure getASCIIz(p: bap; var s: string);
    var i: integer;
  begin { getASCIIz }
    i:= 0; s:= '';
    while p^[i] <> 0 do begin
      insert(chr(p^[i]), s, length(s) +1);
      if length(s) = 255 then begin
        s[0]:= #254;
      end { if };
      inc(i);
    end { while };
  end { getASCIIz };


  procedure Show_BDY_LIB;
    var p: pointer;
        s: string;

  begin { Show_BDY_LIB }
    move(SYSpage^[LIBOffs], (*){$w-}(*) p, 2);
    if p = nil then
      s:= '\SYSTEM.UPL'
    else begin
      getASCIIz(p, s);
    end { if };
    writeln('LIB = ', s);

    (*){$w-}(*) move(SYSpage^[BDYOffs], p, 2);
    if p = nil then
      s:= '.'
    else begin
      getASCIIz(p, s);
    end { if };
    writeln('BDY = ', s);
  end { Show_BDY_LIB };

  procedure GetParameters;
    var i: integer;
        s, s1, s2: string;

    procedure Process(s: string);
      var sp, eq: integer;
          period: boolean;
          s1, s2: string;
    begin { Process }
      while (s <> '') and (s[1] = ' ') do begin
        delete(s, 1, 1);                              { remove leading spaces }
      end { while };

      repeat sp:= pos('  ', s);
        if sp <> 0 then begin                   { remove 2 consecutive spaces }
          delete(s, sp, 1);
        end { if };
      until sp = 0;

      if s = '' then (*) return {*) exit {*};
      sp:= pos(' ', s); eq:= pos('=', s); s1:= s;
      if eq = 0 then begin
        writeln(InvPrm, s); HelpUser;
      end { if };
      if sp = 0 then sp:= eq;
      if sp < eq then begin
        delete(s1, sp, 255);
        delete(s, 1, eq); end
      else begin
        delete(s1, eq, 255);
        delete(s, 1, eq);
      end { if };

      while (s <> '') and (s[1] = ' ') do begin
        delete(s, 1, 1);                              { remove leading spaces }
      end { while };

      while s[length(s)] = ' ' do dec(s[0]);         { remove trailing spaces }

      repeat sp:= pos(' ', s);                       { replace space with ';' }
        if sp <> 0 then begin
          s[sp]:= ';';
        end { if };
      until sp = 0;

      sp:= 0;
      repeat s2:= s; delete(s2, 1, sp);
        eq:= pos(';', s2);
        if eq <> 0 then
          delete(s2, eq, 255)
        else begin
          eq:= length(s2)+1;
        end { if };
        sp:= sp + eq; eq:= pos(':', s2);
        if (eq <> 0) and ((eq <> 2) or not(s2[1] in ['A'..'Z'])) then begin
          writeln(InvPrm, s1, '= ', s); HelpUser;
        end { if };
        delete(s2, 1, eq);
        while s2 <> '' do begin
          period:= false;
          if s2[1] = '\' then
            delete(s2, 1, 1)
          else begin
            while (s2 <> '') and (s2[1] <> '\') do begin
              if (s2[1] = '.') and period or not(s2[1] in Allowed) then begin
                writeln(InvPrm, s1, '= ', s); HelpUser;
              end { if };
              period:= period or (s2[1] = '.'); delete(s2, 1, 1);
            end { while };
          end { if };
        end { while };
      until length(s) < sp;

      if s1 = 'LIB' then begin
        hasLIB:= true;
        if (s[length(s)] in ['\', ':']) or (pos(';', s) <> 0) then begin
          writeln(InvPrm, s1, '= ', s); HelpUser;
        end { if };
        LIBname:= s; end
      else if s1 = 'BDY' then begin
        hasBDY:= true; BDYpaths:= s; end
      else begin
        writeln(InvPrm, s1, '= ', s); HelpUser;
      end { if };
    end { Process };

  begin { GetParameters }
    i:= 0; s1:= ''; s2:= '';
    while i < ParamCount do begin
      inc(i); ParamStr(i, s); UpCaseS(s);
      if s[1] = '=' then begin
        Process(s1); s1:= s2; s2:= s;
        insert(' ', s2, length(s2) +1); end
      else if pos('=', s) <> 0 then begin
        insert(s2, s1, length(s1) +1);
        Process(s1); s1:= ''; s2:= s;
        insert(' ', s2, length(s2) +1); end
      else begin
        insert(s2, s1, length(s1) +1); s2:= s;
        insert(' ', s2, length(s2) +1);
      end { if };
    end { while };
    insert(s2, s1, length(s1) +1);
    if s1 <> '' then begin
      Process(s1);
    end { if };
  end { GetParameters };

begin { BLPath }
  writeln('BLPath. Version 1.60.':-30, '(c) 1990 Software R&D Lab, Sofia.':50);
  SYSpage:= pointer($BE00);
  move(SYSpage^[BDYOffs], (*){$w-}(*) wrk1, 2);
  move(SYSpage^[LIBOffs], (*){$w-}(*) wrk2, 2);
  OldBdy:=''; OldLib:='';
  if wrk1<>nil then GetASCIIz(wrk1,OldBdy);
  if wrk2<>nil then GetASCIIz(wrk2,OldLib);

  move(SYSpage^, Yversion, 2); Yversion:= swap(Yversion);
  Allowed:= ['A'..'Z', '0'..'9', '!', '@', '#', '.', '$',
             '%', '&', '(', ')', '-', '_', '`', '{', '}'];
  (*)
  if Yversion < $0100 + 50 then begin         {     }
    writeln('Inccorect version of Ycode interpreter!');      {  }
    writeln('Version 1.50+ required!');                { - 1.50  }
    halt(2);                                                 {    }
  end { if };
  {*)
  writeln('           ');
  {*}

  if ParamCount = 0 then
    (*) Show_BDY_LIB {*) HelpUser {*}
  else begin
    for Yversion:= 1 to ParamCount do begin           { Yversion used as work }
      ParamStr(Yversion, BDYpaths);                        {   BDYpaths }
      if BDYpaths[1] = '?' then begin           {   ?  HELP   }
        HelpUser;
      end { if };
    end { for };

    BDYpaths:= ''; LIBname:= ''; hasBDY:= false; hasLIB:= false;
    GetParameters;
    if not hasBDY then BDYPaths:=OldBDY;
    if not hasLIB then LibName:=OldLib;

    SpcNeeded:=length(BdyPaths)+length(LibName)+2;
    if SpcNeeded>96 then begin
       writeln('Parameter(s) too long');
       halt(1)
    end;

    if (wrk1=nil) and (wrk2=nil) then wrk1:=blp
    else
    if (wrk1=nil) or ((wrk2<>nil) and
       (cardinal(swap(word(wrk2)))<cardinal(swap(word(wrk1)))))
       then wrk1:=wrk2;

    x:=0;
    if (OldBdy<>'') or hasBDY then PutString(BDYPaths,BDYOffs,x);
    if (OldLib<>'') or hasLIB then PutString(LibName,LIBOffs,x);

    if hasLIB then begin
      writeln('Library is set to: ', LIBname);
    end { if };
    if hasBDY then begin
      writeln('Paths for unit bodies are: ',BDYpaths);
    end { if };
  end { paramcount<>0 };
end { BLPath }.
